@@ -25,6 +25,10 @@
 
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
+#include <gdk/gdkkeysyms.h>
+#ifdef ENABLE_GTK3
+#include <gdk/gdkkeysyms-compat.h>
+#endif
 #include <glib/gi18n.h>
 #include <X11/XKBlib.h>
 
@@ -36,6 +40,7 @@
 
 #include "lxcom.h"
 #include "greeter-utils.h"
+#include "lxcommon.h"
 
 enum {
     COL_SESSION_NAME,
@@ -50,15 +55,16 @@ enum {
     N_LANG_COLS
 };
 
-#define VCONFIG_FILE 		"/var/lib/lxdm/lxdm.conf"
 #define XKB_SYMBOL_DIR		"/usr/share/X11/xkb/symbols.dir"
 
 static GtkBuilder* builder;
 static GKeyFile *config;
 static GKeyFile * var_config;
 static GtkWidget* win;
+static GtkWidget* alignment2;
 static GtkWidget* prompt;
 static GtkWidget* login_entry;
+static GtkWidget* user_list_scrolled;
 static GtkWidget* user_list;
 
 static GtkWidget* sessions;
@@ -110,7 +116,7 @@ static char *get_session_exec(void)
 	GtkTreeModel* model;
 	GtkTreeIter it;
 	gchar *res;
-	if(!lang)
+	if(!sessions)
 		return g_strdup("");
 	
 	if(!gtk_combo_box_get_active_iter(GTK_COMBO_BOX(sessions), &it))
@@ -120,21 +126,93 @@ static char *get_session_exec(void)
 	return res;
 }
 
+static void switch_to_input_user(void)
+{
+	if(user)
+	{
+		g_free(user);
+		user=NULL;
+	}
+	if(pass)
+	{
+		g_free(pass);
+		pass=NULL;
+	}
+	gtk_label_set_text( GTK_LABEL(prompt), _("User:"));
+	gtk_widget_show(prompt);
+	if(user_list)
+	{
+		gtk_widget_hide(login_entry);
+		if(user_list_scrolled)
+			gtk_widget_show(user_list_scrolled);
+		else
+			gtk_widget_hide(user_list);
+		gtk_widget_grab_focus(user_list);
+	}
+	else
+	{
+		gtk_widget_show(login_entry);
+		gtk_widget_grab_focus(login_entry);
+	}
+}
+
+static void switch_to_input_passwd(void)
+{
+	if(user_list!=NULL)
+	{
+		if(user_list_scrolled)
+			gtk_widget_hide(user_list_scrolled);
+		else
+			gtk_widget_hide(user_list);
+	}
+	gtk_label_set_text( GTK_LABEL(prompt), _("Password:") );
+	gtk_entry_set_text(GTK_ENTRY(login_entry), "");
+	gtk_entry_set_visibility(GTK_ENTRY(login_entry), FALSE);
+	gtk_widget_show(login_entry);
+	gtk_widget_grab_focus(login_entry);
+}
+
+static void try_login_user(const char *user)
+{
+	char *session_exec=get_session_exec();
+	char *session_lang=get_session_lang();
+	
+	printf("login user=%s session=%s lang=%s\n",
+			user, session_exec, session_lang);
+			
+	g_free(session_lang);
+	g_free(session_exec);
+			
+}	
+
 static void on_entry_activate(GtkEntry* entry)
 {
 	char* tmp;
 	if( !user )
 	{
 		user = g_strdup( gtk_entry_get_text( GTK_ENTRY(entry) ) );
-		gtk_entry_set_text(GTK_ENTRY(entry), "");
-		gtk_label_set_text( GTK_LABEL(prompt), _("Password:") );
+		if(strlen(user)==0)
+		{
+			g_free(user);
+			user=NULL;
+			return;
+		}
 		if(strchr(user, ' '))
 		{
+			gtk_entry_set_text(GTK_ENTRY(entry), "");
 			g_free(user);
 			user = NULL;
 			return;
 		}
-		gtk_entry_set_visibility(entry, FALSE);
+		if(g_key_file_get_integer(config,"base","skip_password",NULL)!=0)
+		{
+			gtk_label_set_text( GTK_LABEL(prompt), "");
+			try_login_user(user);
+		}
+		else
+		{
+			switch_to_input_passwd();
+		}
 	}
 	else
 	{
@@ -805,7 +883,10 @@ static void on_user_select(GtkIconView *iconview)
 	g_list_foreach (list, (GFunc)gtk_tree_path_free, NULL);
 	g_list_free (list);
 	gtk_tree_model_get(model,&iter,2,&name,-1);
-	gtk_widget_hide(user_list);
+	if(user_list_scrolled)
+		gtk_widget_hide(user_list_scrolled);
+	else
+		gtk_widget_hide(user_list);
 	if(name && name[0])
 	{
 		if(auto_login && is_autologin_user(name))
@@ -825,6 +906,13 @@ static void on_user_select(GtkIconView *iconview)
 			gtk_widget_hide( GTK_WIDGET(login_entry) );
 			return;
 		}
+		if(g_key_file_get_integer(config,"base","skip_password",NULL)!=0)
+		{
+			gtk_label_set_text( GTK_LABEL(prompt), "");
+			user=name;
+			try_login_user(user);
+			return;
+		}
 		gtk_entry_set_text(GTK_ENTRY(login_entry),name);
 		g_free(name);
 		on_entry_activate(GTK_ENTRY(login_entry));
@@ -890,6 +978,10 @@ static gboolean load_user_list(GtkWidget *widget)
 #else
 	gtk_icon_view_set_orientation(GTK_ICON_VIEW(widget),GTK_ORIENTATION_HORIZONTAL);
 #endif
+	// FIXME: this should be done at greeter-gtk3.ui
+	// but set there will cause "Floating point exception"
+	gtk_icon_view_set_columns(GTK_ICON_VIEW(widget),1);
+	
 	model=gtk_list_store_new(5,GDK_TYPE_PIXBUF,G_TYPE_STRING,
 			G_TYPE_STRING,G_TYPE_STRING,G_TYPE_BOOLEAN);
 	gtk_icon_view_set_model(GTK_ICON_VIEW(widget),GTK_TREE_MODEL(model));
@@ -906,18 +998,30 @@ static gboolean load_user_list(GtkWidget *widget)
 	}
 	if(count>3)
 	{
-		// TODO: better ui needed
-		count=3;
+		if(user_list_scrolled)
+		{
+			gtk_alignment_set(GTK_ALIGNMENT(alignment2), 0.5, 0.1, 0, 0.3);
+			gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(user_list_scrolled), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
+		}
+		else
+			count=3;
 	}
 	for(i=0;i<count;i++)
 	{		
 		char *gecos,*face_path,*display;
+		gchar *gecos_escape;
 		gboolean login;
 		GdkPixbuf *face=NULL;
 		gtk_list_store_append(model,&iter);
 		gecos=g_key_file_get_string(kf,users[i],"gecos",0);
 		face_path=g_key_file_get_string(kf,users[i],"face",0);
 		login=g_key_file_get_boolean(kf,users[i],"login",0);
+		if(gecos!=NULL)
+		{
+			char *comma=gecos?strchr(gecos,','):NULL;
+			if (comma)
+				*comma='\0';
+		}
 		if(face_path)
 			face=gdk_pixbuf_new_from_file_at_scale(face_path,48,48,TRUE,NULL);
 		if(!face)
@@ -929,13 +1033,15 @@ static gboolean load_user_list(GtkWidget *widget)
 				face=gtk_icon_theme_load_icon(gtk_icon_theme_get_default(),
 						"avatar-default", 48,GTK_ICON_LOOKUP_FORCE_SIZE,NULL);
 		}
+		gecos_escape=g_markup_escape_text(gecos?gecos:users[i],-1);
 		display=g_strdup_printf("<span font_size=\"x-large\">%s</span>%s%s%s%s",
-			gecos?gecos:users[i],
-			(gecos&&strcmp(gecos,users[i]))?"(":"",
+			gecos_escape,
+			(gecos&&strcmp(gecos,users[i]))?" (":"",
 			(gecos&&strcmp(gecos,users[i]))?users[i]:"",
 			(gecos&&strcmp(gecos,users[i]))?")":"",
 			login?_("\n<i>logged in</i>"):"");
 		// don't translate it now, not freeze
+		g_free(gecos_escape);
 		gtk_list_store_set(model,&iter,0,face,1,display,2,users[i],3,gecos,4,login,-1);
 		if(face) g_object_unref(G_OBJECT(face));
 		g_free(display);
@@ -944,16 +1050,17 @@ static gboolean load_user_list(GtkWidget *widget)
 	}
 	g_strfreev(users);
 	g_key_file_free(kf);
-	
+
 	// add "More ..."
 	gtk_list_store_append(model,&iter);
 	gtk_list_store_set(model,&iter,1,_("More ..."),2,"",3,"",4,FALSE,-1);
-	
+
 	path=gtk_tree_path_new_from_string("0");
 	gtk_icon_view_select_path(GTK_ICON_VIEW(widget),path);
 	gtk_widget_grab_focus(widget);
 	gtk_icon_view_set_cursor(GTK_ICON_VIEW(widget),path,NULL,FALSE);
 	gtk_tree_path_free(path);
+		
 	return TRUE;
 }
 
@@ -981,6 +1088,13 @@ static void on_screen_size_changed(GdkScreen *screen,GtkWidget *win)
 	ui_set_bg(window,config);
 }
 
+static gint login_entry_on_key_press (GtkWidget *widget,GdkEventKey *event)
+{
+	if(event->keyval == GDK_Escape)
+		switch_to_input_user();
+	return FALSE;
+}		     
+
 static void create_win()
 {
     GSList* objs, *l;
@@ -1029,63 +1143,91 @@ static void create_win()
         
     } /* otherwise, let gtk theme paint it. */
 
+    alignment2=(GtkWidget*)gtk_builder_get_object(builder,"alignment2");
+    user_list_scrolled=(GtkWidget*)gtk_builder_get_object(builder,"user_list_scrolled");
     user_list=(GtkWidget*)gtk_builder_get_object(builder,"user_list");
 
     prompt = (GtkWidget*)gtk_builder_get_object(builder, "prompt");
     login_entry = (GtkWidget*)gtk_builder_get_object(builder, "login_entry");
+    if(login_entry!=NULL)
+    {
+		g_signal_connect_after(login_entry,"key-press-event",G_CALLBACK(login_entry_on_key_press),NULL);
+	}
 
     g_signal_connect(login_entry, "activate", G_CALLBACK(on_entry_activate), NULL);
 
-    sessions = (GtkWidget*)gtk_builder_get_object(builder, "sessions");
-    gtk_widget_set_name(sessions, "sessions");
-    fix_combobox_entry(sessions);
-    load_sessions();
-
-    w = (GtkWidget*)gtk_builder_get_object(builder, "bottom_pane");
-    if( g_key_file_get_integer(config, "display", "bottom_pane", 0) )
+    if( g_key_file_get_integer(config, "display", "bottom_pane", 0)==1)
     {
         /* hacks to let GtkEventBox paintable with gtk pixmap engine. */
+        w = (GtkWidget*)gtk_builder_get_object(builder, "bottom_pane");
+        if(g_key_file_get_integer(config, "display", "transparent_pane", 0)==1)
+		{
+		}
+        else
+		{
 #if GTK_CHECK_VERSION(2,18,0)
-        if(gtk_widget_get_app_paintable(w))
+			if(gtk_widget_get_app_paintable(w))
 #else
-        if(GTK_WIDGET_APP_PAINTABLE(w))
+			if(GTK_WIDGET_APP_PAINTABLE(w))
 #endif
 
 #if GTK_CHECK_VERSION(3,0,0)
-		g_signal_connect(w,"draw",G_CALLBACK(on_evt_box_draw),NULL);
+				g_signal_connect(w,"draw",G_CALLBACK(on_evt_box_draw),NULL);
 #else
-		g_signal_connect(w, "expose-event", G_CALLBACK(on_evt_box_expose), NULL);
+				g_signal_connect(w, "expose-event", G_CALLBACK(on_evt_box_expose), NULL);
 #endif
-    }
-    else
-        gtk_event_box_set_visible_window(GTK_EVENT_BOX(w), FALSE);
-
-    if( g_key_file_get_integer(config, "display", "lang", 0) == 0 )
-    {
-        w = (GtkWidget*)gtk_builder_get_object(builder, "lang_box");
-        if( w )
-            gtk_widget_hide(w);
-    }
-    else
-    {
-        lang = (GtkWidget*)gtk_builder_get_object(builder, "lang");
-        gtk_widget_set_name(lang, "lang");
-        fix_combobox_entry(lang);
-        load_langs();
-    }
-    
-    if(g_key_file_get_integer(config, "display", "keyboard", 0)==1)
-    {
-		w=(GtkWidget*)gtk_builder_get_object(builder, "keyboard");
-		if((load_keyboards(w))!=FALSE)
-		{
-			fix_combobox_entry(w);
-			gtk_widget_show(w);
-			w=(GtkWidget*)gtk_builder_get_object(builder, "label_keyboard");
-			if(w) gtk_widget_show(w);
 		}
-	}
+        if( g_key_file_get_integer(config, "display", "hide_sessions", 0)==1)
+        {
+            w = (GtkWidget*)gtk_builder_get_object(builder, "sessions_box");
+            if(w) gtk_widget_hide(w);
+		}
+		else
+		{
+			sessions = (GtkWidget*)gtk_builder_get_object(builder, "sessions");
+			gtk_widget_set_name(sessions, "sessions");
+			fix_combobox_entry(sessions);
+			load_sessions();
+		}
 
+		if( g_key_file_get_integer(config, "display", "lang", 0) == 0 )
+		{
+			w = (GtkWidget*)gtk_builder_get_object(builder, "lang_box");
+			if(w) gtk_widget_hide(w);
+		}
+		else
+		{
+			lang = (GtkWidget*)gtk_builder_get_object(builder, "lang");
+			gtk_widget_set_name(lang, "lang");
+			fix_combobox_entry(lang);
+			load_langs();
+		}
+
+		if(g_key_file_get_integer(config, "display", "keyboard", 0)==1)
+		{
+			w=(GtkWidget*)gtk_builder_get_object(builder, "keyboard");
+			if((load_keyboards(w))!=FALSE)
+			{
+				fix_combobox_entry(w);
+				gtk_widget_show(w);
+				w=(GtkWidget*)gtk_builder_get_object(builder, "label_keyboard");
+				if(w) gtk_widget_show(w);
+			}
+		}
+    }
+    else
+    {
+    	w = (GtkWidget*)gtk_builder_get_object(builder, "bottom_pane");
+	gtk_widget_hide(w);
+    }
+
+    if(g_key_file_get_integer(config, "display", "hide_time", 0)==1)
+    {
+        w = (GtkWidget*)gtk_builder_get_object(builder, "time");
+        gtk_widget_hide(w);
+    }
+    else
+    {
 	if( (w = (GtkWidget*)gtk_builder_get_object(builder, "time"))!=NULL )
 	{
 		guint timeout = g_timeout_add(1000, (GSourceFunc)on_timeout, w);
@@ -1093,14 +1235,22 @@ static void create_win()
 			G_CALLBACK(g_source_remove), GUINT_TO_POINTER(timeout));
 		on_timeout((GtkLabel*)w);
 	}
+    }
 
-	exit_btn = (GtkWidget*)gtk_builder_get_object(builder, "exit");
-	load_exit();
+    if(g_key_file_get_integer(config, "display", "hide_exit", 0)==1)
+    {
+		w=(GtkWidget*)gtk_builder_get_object(builder, "exit");
+        gtk_widget_hide(w);
+    }
+    else
+    {
+		exit_btn = (GtkWidget*)gtk_builder_get_object(builder, "exit");
+		load_exit();
+    }
 
 	ui_get_geometry(window,&rc);
 	gtk_window_move(GTK_WINDOW(win),rc.x,rc.y);
 	gtk_window_set_default_size(GTK_WINDOW(win),rc.width,rc.height);
-	ui_set_bg(window,config);
 
 	if(user_list && !g_key_file_get_integer(config,"userlist","disable",NULL) && 
 			load_user_list(user_list))
@@ -1111,7 +1261,10 @@ static void create_win()
 	{
 		if(user_list)
 		{
-			gtk_widget_hide(user_list);
+			if(user_list_scrolled)
+				gtk_widget_hide(user_list_scrolled);
+			else
+				gtk_widget_hide(user_list);
 			user_list=NULL;
 		}
 	}
@@ -1119,6 +1272,8 @@ static void create_win()
 	ui_add_cursor();
 	ui_set_cursor(gtk_widget_get_window(win),GDK_LEFT_PTR);
 	gtk_widget_show(win);
+	ui_set_bg(window,config);
+	
 	ui_set_focus(window);
 	if(!user_list)
 		gtk_widget_grab_focus(login_entry);
@@ -1142,29 +1297,11 @@ static gboolean on_lxdm_command(GIOChannel *source, GIOCondition condition, gpoi
 	gtk_main_quit();
 	else if( !strncmp(str, "reset", 5) )
 	{
-		if(user)
-		{
-			g_free(user);
-			user=NULL;
-		}
-		if(pass)
-		{
-			g_free(pass);
-			pass=NULL;
-		}
-		gtk_label_set_text( GTK_LABEL(prompt), _("User:"));
-		gtk_widget_show(prompt);
-		if(user_list)
-		{
-			gtk_widget_hide(login_entry);
-			gtk_widget_show(user_list);
-			gtk_widget_grab_focus(user_list);
-		}
-		else
-		{
-			gtk_widget_show(login_entry);
-			gtk_widget_grab_focus(login_entry);
-		}
+		switch_to_input_user();
+	}
+	else if( !strncmp(str, "password", 8))
+	{
+		switch_to_input_passwd();
 	}
 	g_free(str);
 	return TRUE;
